package com.connor.cust.util;

import java.awt.Point;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.poi.ooxml.POIXMLDocumentPart;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.ClientAnchor;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.VerticalAlignment;
import org.apache.poi.ss.util.AreaReference;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellReference;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFColor;
import org.apache.poi.xssf.usermodel.XSSFDrawing;
import org.apache.poi.xssf.usermodel.XSSFFont;
import org.apache.poi.xssf.usermodel.XSSFName;
import org.apache.poi.xssf.usermodel.XSSFPicture;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFShape;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xssf.usermodel.extensions.XSSFCellBorder.BorderSide;

public class POIUtil {

//	public static XSSFColor GRAY = new XSSFColor(new Color(217, 217, 217));
//	public static XSSFColor YELLOW = new XSSFColor(new Color(255, 255, 204));
//	public static XSSFColor BLUE = new XSSFColor(new Color(0, 0, 255));
//	public static XSSFColor GREEN = new XSSFColor(new Color(158, 234, 106));

	public static void setValue(XSSFSheet sheet, int rowId, int col, String val) {
		setValue(sheet, rowId, col, val, null);
	}

	public static int getTitleRow(XSSFSheet sheet, int startRow, String[] titleData) {
		if (sheet == null || titleData.length == 0) {
			return -1;
		}
		int lastRow = sheet.getLastRowNum();
		for (int i = startRow; i <= lastRow; i++) {
			LinkedHashMap<Integer, String> data = POIUtil.readRow(null, sheet, i, false);
			if (data == null) {
				continue;
			}
			boolean match = true;
			for (String title : titleData) {
				if (!data.containsValue(title)) {
					match = false;
					break;
				}
			}
			if (match) {
				return i;
			}
		}
		return -1;
	}

	public static XSSFCellStyle getMergeCellStyle(XSSFWorkbook wb, XSSFColor bgColor, XSSFColor fontColor, String fontFamily, short fontSize, boolean bold, boolean top, boolean right, boolean bottom,
			boolean left, HorizontalAlignment ha) {
		XSSFCellStyle style = wb.createCellStyle();
		style.setBorderBottom(bottom ? BorderStyle.MEDIUM : BorderStyle.THIN);
		style.setBottomBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderLeft(left ? BorderStyle.MEDIUM : BorderStyle.THIN);
		style.setLeftBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderRight(right ? BorderStyle.MEDIUM : BorderStyle.THIN);
		style.setRightBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderTop(top ? BorderStyle.MEDIUM : BorderStyle.THIN);
		style.setTopBorderColor(IndexedColors.BLACK.getIndex());
		style.setAlignment(ha);
		style.setVerticalAlignment(VerticalAlignment.CENTER);
		if (bgColor != null) {
			style.setFillForegroundColor(bgColor);
			style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
		}
		XSSFFont font = wb.createFont();
		if (fontColor != null) {
			font.setColor(fontColor);
		}
		if (!fontFamily.isEmpty()) {
			font.setFontName(fontFamily);// (FontFamily.valueOf(fontFamily));
		}
		if (fontSize > 0) {
			font.setFontHeightInPoints(fontSize);
		}
		font.setBold(bold);
		style.setFont(font);

		return style;
	}

	public static CellRangeAddress CellRangeAddress(XSSFWorkbook wb, XSSFSheet sheet, int a, int b, int c, int d, XSSFColor bgColor, XSSFColor fontColor, String fontFamily, short fontSize,
			boolean bold, boolean top, boolean right, boolean bottom, boolean left, HorizontalAlignment ha) {
		CellRangeAddress res = new CellRangeAddress(a, b, c, d);
		XSSFCellStyle style = getMergeCellStyle(wb, bgColor, fontColor, fontFamily, fontSize, bold, top, right, bottom, left, ha);
		for (int rowId = a; rowId <= b; rowId++) {
			XSSFRow row = sheet.getRow(rowId);
			if (row == null) {
				continue;
			}
			for (int colId = c; colId <= d; colId++) {
				XSSFCell cell = row.getCell(colId);
				if (cell == null) {
					cell = row.createCell(colId);
				}
				cell.setCellStyle(style);
			}
		}
		return res;
	}

	public static CellRangeAddress CellRangeAddress(XSSFWorkbook wb, XSSFSheet sheet, int a, int b, int c, int d, XSSFCellStyle style) {
		CellRangeAddress res = new CellRangeAddress(a, b, c, d);
		for (int rowId = a; rowId <= b; rowId++) {
			XSSFRow row = sheet.getRow(rowId);
			if (row == null) {
				continue;
			}
			for (int colId = c; colId <= d; colId++) {
				XSSFCell cell = row.getCell(colId);
				if (cell == null) {
					cell = row.createCell(colId);
				}
				cell.setCellStyle(style);
			}
		}
		return res;
	}

	public static CellRangeAddress CellRangeAddress(XSSFWorkbook wb, XSSFSheet sheet, int a, int b, int c, int d, XSSFColor bgColor, XSSFColor fontColor, String fontFamily, short fontSize,
			boolean bold, boolean top, boolean right, boolean bottom, boolean left) {
		return CellRangeAddress(wb, sheet, a, b, c, d, bgColor, fontColor, fontFamily, fontSize, bold, top, right, bottom, left, HorizontalAlignment.CENTER);
	}

	public static CellRangeAddress CellRangeAddress(XSSFWorkbook wb, XSSFSheet sheet, int a, int b, int c, int d) {
		return CellRangeAddress(wb, sheet, a, b, c, d, null, null, "SimSun", (short) 11, false, false, false, false, false, HorizontalAlignment.CENTER);
	}

	public static CellRangeAddress CellRangeAddress(XSSFWorkbook wb, XSSFSheet sheet, int a, int b, int c, int d, HorizontalAlignment ha) {
		return CellRangeAddress(wb, sheet, a, b, c, d, null, null, "SimSun", (short) 11, false, false, false, false, false, ha);
	}

	public static void setValue(XSSFSheet sheet, int rowId, int col, String val, XSSFCellStyle style) {
		XSSFRow row = sheet.getRow(rowId);
		if (row == null) {
			row = sheet.createRow(rowId);
		}
		XSSFCell cell = row.getCell(col);
		if (cell == null) {
			cell = row.createCell(col);
		}
		cell.setCellValue(val);
		if (style != null) {
			cell.setCellStyle(style);
		}
	}

	public static void setValue(XSSFRow row, int col, String val, XSSFCellStyle style) {
		XSSFCell cell = row.getCell(col);
		if (cell == null) {
			cell = row.createCell(col);
		}
		cell.setCellValue(val);
		if (style != null) {
			cell.setCellStyle(style);
		}
	}

	public static XSSFCellStyle getCellStyle(XSSFWorkbook wb, XSSFColor color, boolean isRight) {
		XSSFCellStyle style = wb.createCellStyle();
		style.setBorderBottom(BorderStyle.THIN);
		style.setBottomBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderLeft(BorderStyle.THIN);
		style.setLeftBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderRight(isRight ? BorderStyle.MEDIUM : BorderStyle.THIN);
		style.setRightBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderTop(BorderStyle.THIN);
		style.setTopBorderColor(IndexedColors.BLACK.getIndex());
		if (color != null) {
			style.setFillForegroundColor(color);
			style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
		}
		return style;
	}

	public static XSSFCellStyle getCellStyle(XSSFWorkbook wb, XSSFColor color, boolean isRight, String fontFamily, short fontSize) {
		XSSFCellStyle style = wb.createCellStyle();
		style.setBorderBottom(BorderStyle.THIN);
		style.setBottomBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderLeft(BorderStyle.THIN);
		style.setLeftBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderRight(isRight ? BorderStyle.MEDIUM : BorderStyle.THIN);
		style.setRightBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderTop(BorderStyle.THIN);
		style.setTopBorderColor(IndexedColors.BLACK.getIndex());
		if (color != null) {
			style.setFillForegroundColor(color);
			style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
		}
		XSSFFont font = wb.createFont();
		if (!fontFamily.isEmpty()) {
			font.setFontName(fontFamily);// (FontFamily.valueOf(fontFamily));
		}
		if (fontSize > 0) {
			font.setFontHeightInPoints(fontSize);
		}
		style.setFont(font);
		return style;
	}

	public static XSSFCellStyle getCellStyleNoBorder(XSSFWorkbook wb, XSSFColor color, String fontFamily, short fontSize) {
		XSSFCellStyle style = wb.createCellStyle();
		if (color != null) {
			style.setFillForegroundColor(color);
			style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
		}
		XSSFFont font = wb.createFont();
		if (!fontFamily.isEmpty()) {
			font.setFontName(fontFamily);// (FontFamily.valueOf(fontFamily));
		}
		if (fontSize > 0) {
			font.setFontHeightInPoints(fontSize);
		}
		style.setFont(font);
		return style;
	}

	public static XSSFCellStyle getCellStyleCenter(XSSFWorkbook wb, XSSFColor color, boolean isRight, String fontFamily, short fontSize) {
		XSSFCellStyle style = wb.createCellStyle();
		style.setBorderBottom(BorderStyle.THIN);
		style.setBottomBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderLeft(BorderStyle.THIN);
		style.setLeftBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderRight(isRight ? BorderStyle.MEDIUM : BorderStyle.THIN);
		style.setRightBorderColor(IndexedColors.BLACK.getIndex());
		style.setBorderTop(BorderStyle.THIN);
		style.setTopBorderColor(IndexedColors.BLACK.getIndex());
		if (color != null) {
			style.setFillForegroundColor(color);
			style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
		}
		XSSFFont font = wb.createFont();
		if (!fontFamily.isEmpty()) {
			font.setFontName(fontFamily);// (FontFamily.valueOf(fontFamily));
		}
		if (fontSize > 0) {
			font.setFontHeightInPoints(fontSize);
		}
		style.setFont(font);
		style.setVerticalAlignment(VerticalAlignment.CENTER);
		style.setAlignment(HorizontalAlignment.CENTER);
		return style;
	}

	public static void setValue(XSSFRow row, int col, String val) {
		XSSFCell cell = row.getCell(col);
		if (cell == null) {
			cell = row.createCell(col);
			// cell.setCellType(STRING);
		}
		cell.setCellValue(val);
	}
	
	public static void setValue(XSSFRow row, int col, int val) {
		XSSFCell cell = row.getCell(col);
		if (cell == null) {
			cell = row.createCell(col);
		}
		cell.setCellValue(val);
	}
	
	public static void setValue(XSSFRow row, int col, double val) {
		XSSFCell cell = row.getCell(col);
		if (cell == null) {
			cell = row.createCell(col);
		}
		cell.setCellValue(val);
	}

	public static int[] parseLocation(String location) {
		if (location.isEmpty()) {
			return null;
		}
		int ind = -1;
		char[] arr = location.toCharArray();
		int len = arr == null ? 0 : arr.length;
		for (int i = 0; i < len; i++) {
			char c = arr[i];
			if (c >= '0' && c <= '9') {
				ind = i;
				break;
			}
		}
		if (ind < 0) {
			return null;
		}
		String col = location.substring(0, ind);
		String row = location.substring(ind);
		int colNum = parseExcelCol(col) - 1;
		int rowNum = 0;
		if (!row.isEmpty()) {
			try {
				rowNum = Integer.parseInt(row) - 1;
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		return new int[] { colNum, rowNum };
	}

	public static int parseExcelCol(String s) {
		int length = s.length();
		int col = 0;
		for (int i = 0; i < length; i++) {
			col = col * 26 + s.charAt(i) - 64;
		}
		return col;
	}

	public static String formatExcelCol(int s) {
		int n;
		String res = "";
		while (s / 26 != 0) {
			n = s % 26;
			s = s / 26;
			if (n == 0) {
				n = 26;
				s = s - 1;
			}
			res = (char) (n + 64) + res;
		}
		if (s != 0) {
			res = (char) (s + 64) + res;
		}
		return res;
	}

	public static void deleteCol(XSSFSheet sheet, int startRow, int EndRow, int columnToDelete) {
		int maxColumn = 0;
		for (int r = startRow; r <= EndRow; r++) {
			Row row = sheet.getRow(r);
			if (row == null)
				continue;
			int lastColumn = row.getLastCellNum();
			if (lastColumn > maxColumn)
				maxColumn = lastColumn;

			if (lastColumn < columnToDelete)
				continue;
			for (int x = columnToDelete + 1; x < lastColumn + 1; x++) {
				Cell oldCell = row.getCell(x - 1);
				if (oldCell != null)
					row.removeCell(oldCell);

				Cell nextCell = row.getCell(x);
				if (nextCell != null) {
					Cell newCell = row.createCell(x - 1, nextCell.getCellType());
					cloneCell(newCell, nextCell, true);
				}
			}
		}
		for (int c = 0; c < maxColumn; c++) {
			sheet.setColumnWidth(c, sheet.getColumnWidth(c + 1));
		}
	}

	public static void deleteColumn(Sheet sheet, int columnToDelete) {
		int maxColumn = 0;
		for (int r = 0; r < sheet.getLastRowNum() + 1; r++) {
			Row row = sheet.getRow(r);

			// if no row exists here; then nothing to do; next!
			if (row == null)
				continue;

			// if the row doesn't have this many columns then we are good; next!
			int lastColumn = row.getLastCellNum();
			if (lastColumn > maxColumn)
				maxColumn = lastColumn;

			if (lastColumn < columnToDelete)
				continue;

			for (int x = columnToDelete + 1; x < lastColumn + 1; x++) {
				Cell oldCell = row.getCell(x - 1);
				if (oldCell != null)
					row.removeCell(oldCell);

				Cell nextCell = row.getCell(x);
				if (nextCell != null) {
					Cell newCell = row.createCell(x - 1, nextCell.getCellType());
					cloneCell(newCell, nextCell, true);
				}
			}
		}

		// Adjust the column widths
		for (int c = 0; c < maxColumn; c++) {
			sheet.setColumnWidth(c, sheet.getColumnWidth(c + 1));
		}
	}

	/*
	 * Takes an existing Cell and merges all the styles and forumla into the new one
	 */
	public static void cloneCell(Cell cNew, Cell cOld, boolean includeValue) {
		cNew.setCellComment(cOld.getCellComment());
		cNew.setCellStyle(cOld.getCellStyle());
		if (!includeValue) {
			return;
		}
		switch (cOld.getCellType()) {
		case BOOLEAN:
			cNew.setCellValue(cOld.getBooleanCellValue());
			break;
		case NUMERIC:
			cNew.setCellValue(cOld.getNumericCellValue());
			break;
		case STRING:
			cNew.setCellValue(cOld.getStringCellValue());
			break;
		case ERROR:
			cNew.setCellValue(cOld.getErrorCellValue());
			break;
		case FORMULA:
			cNew.setCellFormula(cOld.getCellFormula());
			break;
		default:
			break;
		}

	}

	public static XSSFRow insertRow(XSSFSheet sheet, int row) {
		int regionCnt = sheet.getNumMergedRegions();
		int sourceRowNum = row-1;
		List<CellRangeAddress> copyRanges = new ArrayList<CellRangeAddress>();
		for (int n = 0; n < regionCnt; n++) {
			CellRangeAddress region = sheet.getMergedRegion(n);
			if ((region.getFirstRow() == sourceRowNum) && (region.getLastRow() == sourceRowNum)) {
				copyRanges.add(region);

			}
		}
		XSSFRow sourceRow = sheet.getRow(sourceRowNum);
		sheet.shiftRows(row, sheet.getLastRowNum(), 1, true, false);
		XSSFRow targetRow = null;
		XSSFCell sourceCell = null;
		XSSFCell targetCell = null;
		short m;
		targetRow = sheet.createRow(row);
		if (sourceRow != null) {
			targetRow.setHeight(sourceRow.getHeight());
			for (m = sourceRow.getFirstCellNum(); m < sourceRow.getLastCellNum(); m++) {
				sourceCell = sourceRow.getCell(m);
				targetCell = targetRow.createCell(m);
				targetCell.setCellStyle(sourceCell.getCellStyle());
				targetCell.setCellType(sourceCell.getCellType());
			}
		}
		for (CellRangeAddress region : copyRanges) {
			CellRangeAddress newRegion = new CellRangeAddress(row, row, region.getFirstColumn(), region.getLastColumn());
			sheet.addMergedRegion(newRegion);
		}
		return targetRow;
	}

	/**
	 * Excel
	 * @param workbook 
	 * @param sheet sheetҳ
	 * @param startRow Դʼ
	 * @param endRow ԴֹУ
	 * @param margin Ŀ
	 * @param endCol к
	 * @param copyCount 
	 */
	public static void copyRows(XSSFWorkbook workbook, XSSFSheet sheet, int startRow, int endRow, int margin, int endCol, int copyCount) {
		int regionCnt = sheet.getNumMergedRegions();
		CellRangeAddress region;
		for (int i = startRow; i <= endRow; i++) {
			XSSFRow sourceRow = null;
			XSSFRow targetRow = null;
			XSSFCell sourceCell = null;
			XSSFCell targetCell = null;
			short m;
			sourceRow = getRow(sheet, i);
			for (int j = 0; j < copyCount; j++) {
//				int space = (endRow - startRow + 1 + margin) * j;
//				targetRow = sheet.createRow(i + space);
				int space = (endRow - startRow + 1) * j + margin;
				targetRow = getRow(sheet, i - startRow + space);
				targetRow.setHeight(sourceRow.getHeight());
				for (m = sourceRow.getFirstCellNum(); m <= endCol; m++) {
					if(m < 0)
						continue;
					sourceCell = getCell(sourceRow, m);
					targetCell = targetRow.createCell(m);
					if (sourceCell != null) {
						targetCell.setCellStyle(sourceCell.getCellStyle());
						CellType cType = sourceCell.getCellType();
						switch (cType) {
						case STRING:
							targetCell.setCellType(cType);
							targetCell.setCellValue(sourceCell.getRichStringCellValue());
							break;
						case NUMERIC:
							targetCell.setCellType(cType);
							targetCell.setCellValue(sourceCell.getNumericCellValue());
							break;
						case FORMULA:
							targetCell.setCellFormula(sourceCell.getCellFormula());
							break;
						case BOOLEAN:
							targetCell.setCellType(cType);
							targetCell.setCellValue(sourceCell.getBooleanCellValue());
							break;
						case ERROR:
							targetCell.setCellType(cType);
							targetCell.setCellValue(sourceCell.getErrorCellValue());
							break;
						default:
							targetCell.setCellType(cType);
							targetCell.setCellValue(sourceCell.getRichStringCellValue());
							break;
						}
					}
				}

			}
		}
		for (int j = 0; j < copyCount; j++) {
			int space = margin + (endRow - startRow + 1) * j - startRow;
			for (int n = 0; n < regionCnt; n++) {
				region = sheet.getMergedRegion(n);
				if ((region.getFirstColumn() >= startRow) && (region.getLastRow() <= endRow) && (region.getLastColumn() <= endCol)) {
					CellRangeAddress newRegion = new CellRangeAddress(region.getFirstRow() + space, region.getLastRow() + space, region.getFirstColumn(), region.getLastColumn());
					sheet.addMergedRegion(newRegion);
				}
			}
		}
	}

	public static boolean writeToExcel(File tempFile, int[] cols, String[][] vals, int startRow, int endRow, Map<Point, String> otherData) {
		FileInputStream input = null;
		FileOutputStream output = null;
		int rowCount = vals.length;
		int colCount = cols.length;
		XSSFRow row = null;
		XSSFCell cell = null;
		int index = 0;
		try {
			input = new FileInputStream(tempFile);
			XSSFWorkbook wb = new XSSFWorkbook(input);
			XSSFSheet sheet = wb.getSheetAt(index);
			for (Point point : otherData.keySet()) {
				row = sheet.getRow(point.x);
				cell = row.getCell(point.y);
				cell.setCellType(CellType.STRING);
				cell.setCellValue(otherData.get(point));
			}
			int sheetCount = vals.length % (endRow - startRow + 1) == 0 ? vals.length / (endRow - startRow + 1) : vals.length / (endRow - startRow + 1) + 1;
			Map<Integer, XSSFSheet> sheetMap = new HashMap<>();
			for (int i = 1; i < sheetCount; i++) {
				XSSFSheet newSheet = wb.createSheet();
				copySheet(wb, sheet, newSheet);
				sheetMap.put(i, newSheet);
			}
			for (int i = 0; i < rowCount; i++) {
				if ((i + startRow - (endRow - startRow + 1) * index) > endRow) {
					index++;
					sheet = sheetMap.get(index);
					row = sheet.getRow(i + startRow - (endRow - startRow + 1) * index);
				} else {
					row = sheet.getRow(i + startRow - (endRow - startRow + 1) * index);
				}
				for (int j = 0; j < colCount; j++) {
					cell = row.getCell(cols[j]);
					cell.setCellType(CellType.STRING);
					cell.setCellValue(vals[i][j]);
				}
			}
			output = new FileOutputStream(tempFile);
			wb.write(output);
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		} finally {
			if (output != null) {
				try {
					output.close();
					input.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		System.out.println("-------WRITE EXCEL OVER-------");
		return true;
	}

	public static XSSFSheet copySheet(XSSFWorkbook workbook, XSSFSheet sheetFrom, XSSFSheet sheetTo) {
		CellRangeAddress region = null;
		XSSFRow rowFrom = null;
		XSSFRow rowTo = null;
		XSSFCell cellFrom = null;
		XSSFCell cellTo = null;
		for (int i = 0; i < sheetFrom.getNumMergedRegions(); i++) {
			region = sheetFrom.getMergedRegion(i);
			if ((region.getFirstColumn() >= sheetFrom.getFirstRowNum()) && (region.getLastRow() <= sheetFrom.getLastRowNum())) {
				sheetTo.addMergedRegion(region);
			}
		}
		for (int intRow = sheetFrom.getFirstRowNum(); intRow < sheetFrom.getLastRowNum(); intRow++) {
			rowFrom = sheetFrom.getRow(intRow);
			rowTo = sheetTo.createRow(intRow);
			if (null == rowFrom)
				continue;
			rowTo.setHeight(rowFrom.getHeight());
			for (int intCol = 0; intCol < rowFrom.getLastCellNum(); intCol++) {
				sheetTo.setDefaultColumnStyle(intCol, sheetFrom.getColumnStyle(intCol));
				sheetTo.setColumnWidth(intCol, sheetFrom.getColumnWidth(intCol));
				cellFrom = rowFrom.getCell(intCol);
				cellTo = rowTo.createCell(intCol);
				if (null == cellFrom)
					continue;
				cellTo.setCellStyle(cellFrom.getCellStyle());
				CellType cType = cellFrom.getCellType();
				cellTo.setCellType(cType);
				switch (cType) {
				case STRING:
					cellTo.setCellValue(cellFrom.getRichStringCellValue());
					break;
				case NUMERIC:
					cellTo.setCellValue(cellFrom.getNumericCellValue());
					break;
				case FORMULA:
					cellTo.setCellFormula(cellFrom.getCellFormula());
					break;
				case BOOLEAN:
					cellTo.setCellValue(cellFrom.getBooleanCellValue());
					break;
				case ERROR:
					cellTo.setCellValue(cellFrom.getErrorCellValue());
					break;
				default:
					cellTo.setCellValue(cellFrom.getRichStringCellValue());
					break;
				}
				/*
				 * if (null != cellFrom.getStringCellValue() &&
				 * !"".equals(cellFrom.getStringCellValue().trim()))
				 * cellTo.setCellValue(cellFrom.getStringCellValue());
				 */
			}
		}
		XSSFDrawing patriarchTo = sheetTo.createDrawingPatriarch();
		for (POIXMLDocumentPart dr : sheetFrom.getRelations()) {
			if (dr instanceof XSSFDrawing) {
				XSSFDrawing drawing = (XSSFDrawing) dr;
				List<XSSFShape> shapes = drawing.getShapes();
				for (XSSFShape shape : shapes) {
					XSSFPicture pic = (XSSFPicture) shape;
					patriarchTo.createPicture((ClientAnchor) pic.getAnchor(), workbook.addPicture(pic.getPictureData().getData(), XSSFWorkbook.PICTURE_TYPE_PNG));
				}
			}
		}
		sheetTo.setDisplayGridlines(false);
		sheetTo.setZoom(100);
		return sheetTo;

	}

	// public static String getValue(XSSFRow titleRow, int i) {
	// XSSFCell cell = titleRow.getCell(i);
	// if(cell==null) {
	// return "";
	// }
	// if(cell.getCellType()==NUMERIC) {
	// return cell.getNumericCellValue()+"";
	// }else {
	// return cell.getStringCellValue();
	// }
	// }

	private static DecimalFormat DF = new DecimalFormat("#.##########################################");

	public static String getValue(XSSFRow titleRow, int i) {
		XSSFCell cell = titleRow.getCell(i);
		if (cell == null) {
			return "";
		}
		if (cell.getCellType() == CellType.NUMERIC) {
			return DF.format(cell.getNumericCellValue());
		} else if (cell.getCellType() == CellType.STRING) {
			return cell.getStringCellValue();
		}
		return "";
	}

	public static String getValue(XSSFSheet sheet, int i, int j) {
		XSSFRow row = sheet.getRow(i);
		if (row == null) {
			return null;
		}
		return getValue(row, j);
	}

	public static XSSFRow getRow(XSSFSheet sheet, int i) {
		XSSFRow row = sheet.getRow(i);
		if (row == null) {
			row = sheet.createRow(i);
		}
		return row;
	}

	public static LinkedHashMap<Integer, String> readRow(SimpleDateFormat dateFormat, XSSFSheet sheet, int rowNum, boolean readEmpty) {
		if (sheet == null) {
			return null;
		}
		XSSFRow row = sheet.getRow(rowNum);
		if (row == null) {
			return null;
		}
		LinkedHashMap<Integer, String> data = new LinkedHashMap<Integer, String>();
		int firstCol = row.getFirstCellNum();
		if (firstCol < 0) {
			firstCol = 0;
		}
		int lastCol = row.getLastCellNum();
		for (int i = firstCol; i <= lastCol; i++) {
			XSSFCell cell = row.getCell(i);
			if (cell == null) {
				continue;
			}
			String value;
			switch (cell.getCellType()) {
			case STRING:
				value = cell.getStringCellValue();
				break;
			case NUMERIC:
				if (dateFormat != null && DateUtil.isCellDateFormatted(cell)) {
					value = cell.getDateCellValue() == null ? "" : dateFormat.format(cell.getDateCellValue());
				} else {
					value = DF.format(cell.getNumericCellValue());
				}
				break;
			case FORMULA:
				value = cell.getCellFormula();
				break;
			case ERROR:
				value = cell.getErrorCellString();
				break;
			case BOOLEAN:
				value = cell.getBooleanCellValue() ? "true" : "false";
				break;
			case BLANK:
				value = "";
				break;
			default:
				value = "";
				break;
			}
			if (!readEmpty && (value == null || "".equals(value))) {
				continue;
			}
			data.put(i, value);
		}
		return data;
	}

	public static LinkedHashMap<Integer, String> readRow(SimpleDateFormat dateFormat, XSSFSheet sheet, int rowNum, Integer[] cols) {
		if (sheet == null || cols.length == 0) {
			return null;
		}
		XSSFRow row = sheet.getRow(rowNum);
		if (row == null) {
			return null;
		}
		LinkedHashMap<Integer, String> data = new LinkedHashMap<Integer, String>();
		for (Integer i : cols) {
			XSSFCell cell = row.getCell(i);
			if (cell == null) {
				data.put(i, "");
				continue;
			}
			String value;
			switch (cell.getCellType()) {
			case STRING:
				value = cell.getStringCellValue();
				break;
			case NUMERIC:
				if (dateFormat != null && DateUtil.isCellDateFormatted(cell)) {
					value = cell.getDateCellValue() == null ? "" : dateFormat.format(cell.getDateCellValue());
				} else {
					value = DF.format(cell.getNumericCellValue());
				}
				break;
			case FORMULA:
				value = cell.getCellFormula();
				break;
			case ERROR:
				value = cell.getErrorCellString();
				break;
			case BOOLEAN:
				value = cell.getBooleanCellValue() ? "true" : "false";
				break;
			case BLANK:
				value = "";
				break;
			default:
				value = "";
				break;
			}
			data.put(i, value);
		}
		return data;
	}

	public static void writeToNamedCells(XSSFWorkbook wb, Map<String, String> excelDataMap) throws Exception {
		if (wb == null || excelDataMap == null || excelDataMap.size() == 0) {
			return;
		}
		for (String cellName : excelDataMap.keySet()) {
			String value = excelDataMap.get(cellName);
			System.out.println("[" + cellName + "]=[" + value + "]");
			XSSFName name = wb.getName(cellName);
			if (name == null) {
				System.out.println(">> Named Cell Not Found");
			} else {
				AreaReference aref = new AreaReference(name.getRefersToFormula(), null);
				CellReference[] crefs = aref.getAllReferencedCells();
				for (int i = 0; i < crefs.length; i++) {
					XSSFSheet s = wb.getSheet(crefs[i].getSheetName());
					int row = crefs[i].getRow();
					int col = crefs[i].getCol();
					System.out.println(">> " + crefs[i].formatAsString());
					POIUtil.setValue(s, row, col, value);
				}
			}
		}
	}

	public static XSSFCellStyle cloneCellStype(XSSFCellStyle fromStyle, XSSFWorkbook wb, Map<XSSFFont, XSSFFont> fontMap) throws Exception{
		if(fromStyle==null||wb==null) {
			return null;
		}
		XSSFCellStyle toStyle = wb.createCellStyle();
		toStyle.setAlignment(fromStyle.getAlignment());
		toStyle.setBorderColor(BorderSide.TOP, fromStyle.getBorderColor(BorderSide.TOP));
		toStyle.setBorderColor(BorderSide.LEFT, fromStyle.getBorderColor(BorderSide.LEFT));
		toStyle.setBorderColor(BorderSide.BOTTOM, fromStyle.getBorderColor(BorderSide.BOTTOM));
		toStyle.setBorderColor(BorderSide.RIGHT, fromStyle.getBorderColor(BorderSide.RIGHT));
		toStyle.setBorderTop(fromStyle.getBorderTop());
		toStyle.setBorderLeft(fromStyle.getBorderLeft());
		toStyle.setBorderBottom(fromStyle.getBorderBottom());
		toStyle.setBorderRight(fromStyle.getBorderRight());
		XSSFFont fromFont = fromStyle.getFont();
		if(fromFont!=null) {
			XSSFFont toFont = wb.createFont();
			if(fontMap.containsKey(fromFont)) {
				toFont = fontMap.get(fromFont);
			}else {
				toFont = wb.createFont();
				toFont.setBold(fromFont.getBold());
//				toFont.setBoldweight(fromFont.getBoldweight());
				toFont.setCharSet(fromFont.getCharSet());
				toFont.setColor(fromFont.getColor());
				toFont.setColor(fromFont.getXSSFColor());
				toFont.setFamily(fromFont.getFamily());
				toFont.setFontHeight(fromFont.getFontHeight());
				toFont.setFontHeightInPoints(fromFont.getFontHeightInPoints());
				toFont.setFontName(fromFont.getFontName());
				toFont.setItalic(fromFont.getItalic());
				toFont.setScheme(fromFont.getScheme());
				toFont.setStrikeout(fromFont.getStrikeout());
				// toFont.setThemeColor(fromFont.getThemeColor());
				toFont.setTypeOffset(fromFont.getTypeOffset());
				toFont.setUnderline(fromFont.getUnderline());
			}
			toStyle.setFont(toFont);
		}
		toStyle.setFillBackgroundColor(fromStyle.getFillBackgroundXSSFColor());
		toStyle.setFillForegroundColor(fromStyle.getFillForegroundXSSFColor());
		toStyle.setFillPattern(fromStyle.getFillPattern());
		toStyle.setHidden(fromStyle.getHidden());
		toStyle.setIndention(fromStyle.getIndention());
		toStyle.setLocked(fromStyle.getLocked());
		toStyle.setRotation(fromStyle.getRotation());
		toStyle.setVerticalAlignment(fromStyle.getVerticalAlignment());
		toStyle.setWrapText(fromStyle.getWrapText());
		return toStyle;
	}

	/**
	 * ȡExcelԪ
	 * @param row Ԫ
	 * @param col Ԫк
	 * @return ExcelԪ
	 */
	private static XSSFCell getCell(XSSFRow row, int col) {
		XSSFCell cell = row.getCell(col);
		if(cell==null)
			cell = row.createCell(col);
		return cell;
	}


	/**
	 * Excel
	 */
	public static void main(String[] args) {
        System.out.println("BoxReport main");
		FileInputStream input = null;
		FileOutputStream output = null;
		XSSFWorkbook workbook = null;
        try {
        	String s = "[b][c[";
        	String[] ss = s.split("[\\[\\]]");
            System.out.println(Arrays.toString(ss));
        	/*File file = new File("Z:\\TC_install\\SGXY\\ҳģ0 - .xlsx");
			input = new FileInputStream(file);
			workbook = new XSSFWorkbook(input);
			XSSFSheet sheet = workbook.getSheet("װϸ");
//			for(int i=0; i<wb.getNumberOfSheets(); i++) {
//				if(wb.getSheetAt(i).getSheetName().equals("װϸ")) {
//					sheet = wb.getSheetAt(i);
//					sheet0 = i;
//					break;
//				}
//			}
			if(sheet == null) {
		        System.out.println("sheeterr");
	            return;
			}
			int sheet0 = workbook.getSheetIndex(sheet);
	        System.out.println("sheet0:"+sheet0);
			CellCopyPolicy cpp = new CellCopyPolicy();
			cpp.setCopyCellFormula(true);
			cpp.setCopyCellStyle(true);
			cpp.setCopyCellValue(true);
			cpp.setCopyMergedRegions(true);
			cpp.setCopyRowHeight(true);
			cpp.setMergeHyperlink(true);
			for(int i=0; i<2; i++) {
				XSSFSheet sheet2 = workbook.cloneSheet(sheet0);
				workbook.setSheetOrder(sheet2.getSheetName(), sheet0+i+1);
				XSSFPrintSetup ps = sheet2.getPrintSetup();
				ps.setLandscape(true);
				ps.setPaperSize(sheet.getPrintSetup().getPaperSizeEnum());
//				wb.
//				for(int r=i*27; r<i*27+27; r++) {
//					XSSFRow xssfrow = TCUtil.getRow(sheet, r);
//					for(int c=0; c<10; c++) {
//						TCUtil.getCell(xssfrow, c);
//					}
//				}
//				sheet.copyRows(0, 27, i*28, cpp);
//				sheet.copyRows(8, 9, 28, new CellCopyPolicy());
//				copyRows(wb, sheet, 0, 26, i*27, 9, 1);
			}

			output = new FileOutputStream(file.getPath());
			workbook.write(output);*/
	        System.out.println("end");
        } catch (Exception e1) {
            e1.printStackTrace();
        }finally {
        	try {
    			if(workbook != null)
    				workbook.close();
    			if(output != null)
    				output.close();
    			if(input != null)
    				input.close();
            } catch (Exception e1) {
                e1.printStackTrace();
            }
		}
	}
}
